@@ -19,6 +19,10 @@ from tempest.lib import exceptions as lib_exc
 import testtools
 
 from neutron_tempest_plugin.api import base_routers as base
+from neutron_tempest_plugin import config
+
+
+CONF = config.CONF
 
 
 class RoutersNegativeTestBase(base.BaseRouterTest):
@@ -67,6 +71,44 @@ class RoutersNegativePolicyTest(RoutersNegativeTestBase):
             client2.add_router_interface_with_subnet_id(
                 self.router['id'], subnet['id'])
 
+    @decorators.attr(type='negative')
+    @decorators.idempotent_id('8010d27e-4ab7-4ea3-98b1-3995b7910efd')
+    def test_add_interface_in_use(self):
+        port = self.create_port(self.network)
+        self.client.add_router_interface_with_port_id(
+            self.router['id'], port['id'])
+        self.assertRaises(
+            lib_exc.Conflict,
+            self.client.add_router_interface_with_port_id,
+            self.router['id'], port['id'])
+
+    @decorators.attr(type='negative')
+    @decorators.idempotent_id('ed84c800-ee29-4b76-9419-d6d7b143fc47')
+    def test_add_interface_port_nonexist(self):
+        # port id is not a valid UUID
+        invalid_id = data_utils.rand_name('port')
+        self.assertRaises(
+            lib_exc.BadRequest,
+            self.client.add_router_interface_with_port_id,
+            self.router['id'], invalid_id)
+
+    @decorators.attr(type='negative')
+    @decorators.idempotent_id('dad7a8ba-2726-11eb-82dd-74e5f9e2a801')
+    def test_remove_associated_ports(self):
+        self.client.update_router(
+            self.router['id'],
+            external_gateway_info={
+                'network_id': CONF.network.public_network_id})
+        network = self.create_network()
+        subnet = self.create_subnet(network)
+        self.create_router_interface(self.router['id'], subnet['id'])
+        port_ids = [
+            item['id'] for item in self.admin_client.list_ports(
+                device_id=self.router['id'])['ports']]
+        for port_id in port_ids:
+            with testtools.ExpectedException(lib_exc.Conflict):
+                self.admin_client.delete_port(port_id)
+
 
 class DvrRoutersNegativeTest(RoutersNegativeTestBase):
 
